@@ -558,3 +558,19 @@ int nids_next()
     pcap_hand(0, &h, data);
     return 1;
 }
+
+int nids_dispatch(int cnt)
+{
+    int r;
+
+    if (!desc) {
+	strcpy(nids_errbuf, "Libnids not initialized");
+	return -1;
+    }
+    if ((r = pcap_dispatch(desc, cnt, (pcap_handler) pcap_hand,
+                                    NULL)) == -1) {
+	strcpy(nids_errbuf, "dispatch: ");
+	strncat(nids_errbuf, pcap_geterr(desc), sizeof(nids_errbuf) - 11);
+    }
+    return r;
+}
