@@ -145,6 +145,37 @@ static int get_ts(struct tcphdr * this_tcphdr, unsigned int * ts)
 			
   return ret;
 }  		
+
+static int get_wscale(struct tcphdr * this_tcphdr, unsigned int * ws)
+{
+  int len = 4 * this_tcphdr->th_off;
+  unsigned int tmp_ws;
+  unsigned char * options = (char*)(this_tcphdr + 1);
+  int ind = 0, ret = 0;
+  *ws=1;
+  while (ind <=  len - (int)sizeof (struct tcphdr) - 3 )
+  	switch (options[ind]) {
+		case 0: /* TCPOPT_EOL */
+			return ret;
+		case 1: /* TCPOPT_NOP */
+			ind++;
+			continue;	
+  		case 3: /* TCPOPT_WSCALE */
+  			tmp_ws=options[ind+2];
+  			if (tmp_ws>14) 
+  				tmp_ws=14;
+			*ws=1<<tmp_ws;
+			ret = 1;
+			/* no break, intentionally */
+		default:	
+			if (options[ind+1] < 2 ) /* "silly option" */
+				return ret;
+			ind += options[ind+1];
+	}			
+			
+  return ret;
+}  		
+
     
 
 
@@ -188,6 +219,7 @@ add_new_tcp(struct tcphdr * this_tcphdr, struct ip * this_iphdr)
   a_tcp->client.first_data_seq = a_tcp->client.seq;
   a_tcp->client.window = ntohs(this_tcphdr->th_win);
   a_tcp->client.ts_on = get_ts(this_tcphdr, &a_tcp->client.curr_ts);
+  a_tcp->client.wscale_on = get_wscale(this_tcphdr, &a_tcp->client.wscale);
   a_tcp->server.state = TCP_CLOSE;
   a_tcp->next_node = tolink;
   a_tcp->prev_node = 0;
@@ -502,15 +534,16 @@ handle_ack(struct half_stream * snd, u_int acknum)
     snd->ack_seq = acknum;
   }
 }
-
+#if 0
 static void
 check_flags(struct ip * iph, struct tcphdr * th)
 {
   u_char flag = *(((u_char *) th) + 13);
-  
   if (flag & 0x40 || flag & 0x80)
     nids_params.syslog(NIDS_WARN_TCP, NIDS_WARN_TCP_BADFLAGS, iph, th);
+//ECN is really the only cause of these warnings...
 }
+#endif
 
 struct tcp_stream *
 find_stream(struct tcphdr * this_tcphdr, struct ip * this_iphdr,
@@ -607,8 +640,10 @@ process_tcp(u_char * data, int skblen)
 		       this_tcphdr);
     return;
   }
-
+#if 0
   check_flags(this_iphdr, this_tcphdr);
+//ECN
+#endif
   if (!(a_tcp = find_stream(this_tcphdr, this_iphdr, &from_client))) {
     if ((this_tcphdr->th_flags & TH_SYN) &&
 	!(this_tcphdr->th_flags & TH_ACK) &&
@@ -640,12 +675,23 @@ process_tcp(u_char * data, int skblen)
 	if (!a_tcp->server.ts_on)
 		a_tcp->client.ts_on = 0;
     } else a_tcp->server.ts_on = 0;	
+    if (a_tcp->client.wscale_on) {
+    	a_tcp->server.wscale_on = get_wscale(this_tcphdr, &a_tcp->server.wscale);
+	if (!a_tcp->server.wscale_on) {
+		a_tcp->client.wscale_on = 0;
+		a_tcp->client.wscale  = 1;
+		a_tcp->server.wscale = 1;
+	}	
+    } else {
+    	a_tcp->server.wscale_on = 0;	
+    	a_tcp->server.wscale = 1;
+    }	
     return;
   }
   if (
   	! (  !datalen && ntohl(this_tcphdr->th_seq) == rcv->ack_seq  )
   	&&
-  	( !before(ntohl(this_tcphdr->th_seq), rcv->ack_seq + rcv->window) ||
+  	( !before(ntohl(this_tcphdr->th_seq), rcv->ack_seq + rcv->window*rcv->wscale) ||
           before(ntohl(this_tcphdr->th_seq) + datalen, rcv->ack_seq)  
         )
      )     
